/* cc -Wall -g -pedantic XRandRfl.cxx -o XRandRfl -lX11 `pkg-config --libs xrandr` */
#include <X11/extensions/Xrandr.h>
#include <X11/Xlib.h>
#include <stdio.h>
#include <string.h>

int main(int argc, char ** argv)
{
  Display * display = XOpenDisplay(":0.0");
  int screen = DefaultScreen( display );
  Window w = RootWindow(display, screen);
  float xft_dpi = 0.0;

  // check Xft.dpi settings as used by DE's and apps 
  // tested with KDE
  char * xdefs = XResourceManagerString (display);
  if(xdefs)
  {
    const char * xft = strstr(xdefs, "Xft.dpi:\t");
    if(xft)
    {
      sscanf( xft, "Xft.dpi:\t%f", &xft_dpi );
      fprintf( stdout, "XResource Xft.dpi: %.02f\n", xft_dpi );
    }
  }

  if(xft_dpi == 0)
  {
    XRRScreenResources * res = XRRGetScreenResources(display, w);
    for(int i=0; i < res->noutput; ++i)
    {
      XRROutputInfo * output_info = XRRGetOutputInfo( display, res, res->outputs[i]);
      if(output_info->crtc)
      {
        XRRCrtcInfo * crtc_info = XRRGetCrtcInfo( display, res,
                                                  output_info->crtc );
        unsigned int pixel_width = crtc_info->width,
                     pixel_height = crtc_info->height;
        float xdpi = pixel_width * 25.4f / (float)output_info->mm_width,
              ydpi = pixel_height * 25.4f / (float)output_info->mm_height;
        printf( "[%d] %upx x %upx  Dimensions: %limm x %limm  DPI: %.02f x %.02f\n",
                i, pixel_width, pixel_height,
                output_info->mm_width, output_info->mm_height,
                xdpi, ydpi );
      }
      XRRFreeOutputInfo( output_info );
    }
    XRRFreeScreenResources(res);
  }

  XCloseDisplay( display );
  return 0;
}
